#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif

#include <iostream>

#include "itkComposeUniformTransformsTransform.h"
#include "itkVectorContainer.h"
#include "vnl/vnl_vector_fixed.h"
#include "itkPermuteAxesTransform.h"

int itkComposeUniformTransformsTransformTest(int ,char *[] )
{

  int i;

  /* Create two non-commutative permutations. */
  typedef itk::PermuteAxesTransform<double,3>  InnerTransformType;
  InnerTransformType::Pointer t1 = InnerTransformType::New();
  InnerTransformType::Pointer t2 = InnerTransformType::New();
  InnerTransformType::PermuteOrderArrayType order1, order2;
  order1[0] = 1; order1[1] = 2; order1[2] = 0;
  order2[0] = 1; order2[1] = 0; order2[2] = 2;
  t1->SetOrder(order1);
  t2->SetOrder(order2);


  /* Initialize empty transform and test it. */
  typedef itk::VectorContainer<int,InnerTransformType::Pointer> ContainerType;
  typedef itk::ComposeUniformTransformsTransform<double,3,ContainerType> TransformType;
  TransformType::Pointer transform = TransformType::New();
  ContainerType::Pointer container = ContainerType::New();

  if (transform->GetTransforms()->Size() != 0)
    {
    std::cout << "Invalid initialization." << std::endl;
    return EXIT_FAILURE;
    }

  /* Transform point */
  itk::Point<double, 3> p;
  p[0] = 1; p[1] = 2; p[2] = 3;
  if (p != transform->TransformPoint(p))
    { 
    std::cout << "Invalid transform point by empty list." << std::endl;
    return EXIT_FAILURE;
    }

  /* Transform vector */
  itk::Vector<double, 3> v;
  v[0] = 1; v[1] = 2; v[2] = 3;
  if (v != transform->TransformVector(v))
    { 
    std::cout << "Invalid transform vector by empty list." << std::endl;
    return EXIT_FAILURE;
    }

  /* Transform vnl_vector */
  vnl_vector_fixed<double, 3> n;
  n[0] = 1; n[1] = 2; n[2] = 3;
  if (n != transform->TransformVector(n))
    { 
    std::cout << "Invalid transform vnl_vector by empty list." << std::endl;
    return EXIT_FAILURE;
    }

  /* Transform covariant vector */
  itk::CovariantVector<double, 3> c;
  c[0] = 1; c[1] = 2; c[2] = 3;
  if (c != transform->TransformCovariantVector(c))
    { 
    std::cout << "Invalid transform covariant vector by empty list." << std::endl;
    return EXIT_FAILURE;
    }
 
  /* Initialize a list of transforms and test it */
  ContainerType::ConstPointer container2;
  container->InsertElement(0,t1);
  container->InsertElement(1,t2);
  transform->SetTransforms(container);
  container2 = transform->GetTransforms();
  try 
    {
    int len = (container->Size()>container2->Size())?container->Size():container2->Size();
    for ( i = 0; i < len ; i++)
      if ( container->ElementAt(i)->GetOrder() != container2->ElementAt(i)->GetOrder())
        {
        std::cout << "Passed and returned list of transforms differ.";
        return EXIT_FAILURE;
        }
    }
  catch(...)
    {
    std::cout << "Passed and returned list of transforms differ.";
    return EXIT_FAILURE;
    }

  /* Transform point */
  if ( t2->TransformPoint(t1->TransformPoint(p)) != transform->TransformPoint(p))
    { 
    std::cout << "Invalid transform point." << std::endl;
    return EXIT_FAILURE;
    }

  /* Transform vector */
  if ( t2->TransformVector(t1->TransformVector(v)) != transform->TransformVector(v))
    { 
    std::cout << "Invalid transform vector." << std::endl;
    return EXIT_FAILURE;
    }

  /* Transform vnl_vector */
  if ( t2->TransformVector(t1->TransformVector(n)) != transform->TransformVector(n))
    { 
    std::cout << "Invalid transform vnl_vector." << std::endl;
    return EXIT_FAILURE;
    }

  /* Transform covariant vector */
  if ( t2->TransformCovariantVector(t1->TransformCovariantVector(c)) != transform->TransformCovariantVector(c))
    { 
    std::cout << "Invalid transform covariant vector." << std::endl;
    return EXIT_FAILURE;
    }
 
  return EXIT_SUCCESS;
}
